@@ -3,9 +3,8 @@ package CallBackery::User;
 # $Id: User.pm 539 2013-12-09 22:28:11Z oetiker $
 
 # sorted hashes
-use Mojo::Base -base;
+use Mojo::Base -base, -signatures;
 use Carp qw(croak confess);
-use Scalar::Util qw(weaken);
 use Mojo::Util qw(b64_decode b64_encode secure_compare);
 use Mojo::JSON qw(encode_json decode_json);
 use CallBackery::Exception qw(mkerror);
@@ -35,7 +34,7 @@ the controller
 
 =cut
 
-has 'controller';
+has 'controller', undef, weak => 1;
 
 =head2 $self->userId
 
@@ -80,7 +79,7 @@ a handle to a L<CallBackery::Database> object.
 has app => sub {
     my $app = shift->controller->app;
     return $app;
-};
+}, weak => 1;
 
 has log => sub {
     shift->app->log;
@@ -278,13 +277,22 @@ sub makeSessionCookie {
     return $conf.':'.$check;
 }
 
-sub DESTROY {
-    local($., $@, $!, $^E, $?);
-    return if ${^GLOBAL_PHASE} eq 'DESTRUCT';
-    my $self = shift;
-    $self->log->debug("Destroying ".__PACKAGE__);
+sub DESTROY ($self) {
+    # we are only interested in objects that get destroyed during
+    # global destruction as this is a potential problem
+    my $class = ref($self) // "child of ". __PACKAGE__;
+    if (${^GLOBAL_PHASE} ne 'DESTRUCT') {
+        # $self->log->debug($class." DESTROYed");
+        return;
+    }
+    if ($self && ref $self->log){
+        $self->log->warn("late destruction of $class object during global destruction") unless $self->{prototype};
+        return;
+    }
+    warn "extra late destruction of $class object during global destruction\n" unless ref $self and $self->{prototype};
 }
 
+
 1;
 __END__
 
